{
    This file is part of CheSkymp,
    Copyright (C) 2018 ChebMaster.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; If not, see http://www.gnu.org/licenses/

 **********************************************************************}


{$typeinfo on}
{$ifdef windows}
   {$apptype console}
   //Without this, the very first WriteLn would crash with an I/O error
{$endif}


program cheskymp;

{$unitpath ./Imaging/Source;./Imaging/Source/ZLib}
{$includepath ./Imaging/Source}

{$mode objfpc}
{$longstrings on}

uses
  //lineinfo,
  sysutils, math, skymp;

type
  TMySkymp = class (TSkymp)
  public
    procedure OutStr(s: Ansistring); override;
    procedure ProgressCallback(p: float); override;
  end;

  procedure TMySkymp.OutStr(s: Ansistring);
  begin
    WriteLn(s);
  end;


  procedure TMySkymp.ProgressCallback(p: float);
  var curprogress: integer;
  begin
    curprogress:= round(p * 63);
    if curprogress = prevprogress then Exit;
    if curprogress = 0 then WriteLn;
    prevprogress:= curprogress;
    Write('*');
  end;

var
  S: TMySkymp;
  filename: string;

  func,source: shortstring;
  line: longint;

begin
  filename:= ParamStr(1);
  if filename = '' then begin
    Writeln('FILE NAME NOT SPECIFIED!');
    Writeln('Usage: >cheskymp <file name>');
    Halt(1);
  end;
  try
    s:= TMySkymp.Create(filename);
    if not s.execute
      then WriteLn('FAILED.');
    s.Free;
  except
    WriteLn;
    WriteLn('ERROR ---------------------------------');
    WriteLn((ExceptObject as Exception).Message);
    if not ((ExceptObject as Exception) is ESkympDie) then begin
      if GetLineInfo(ptruint(ExceptAddr), func, source, line) then begin
        WriteLn(ExceptObject.Classname, ' at ', ExtractFileName(source), ':', IntToStr(line), ' (', LowerCase(func), ')');
      end
      else
        WriteLn(ExceptObject.Classname, ' at ', IntToHex(ptruint(ExceptAddr), 8));
    end;
    Halt(1);
  end;
end.

