{
    This file is part of CheSkymp,
    Copyright (C) 2018 ChebMaster.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; If not, see http://www.gnu.org/licenses/

 **********************************************************************}



unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls, IniFiles, lineinfo, math, skymp;

type

  TMySkymp = class;

  { TForm1 }

  TForm1 = class(TForm)
    ButtonRepeat: TButton;
    LoadButton: TButton;
    Memo1: TMemo;
    OpenDialog1: TOpenDialog;
    ProgressBar1: TProgressBar;
    procedure ButtonExecuteClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
  private
    ini: TIniFile;
    dir, filename: string;
    skymp: TMySkymp;
    procedure Run;
    { private declarations }
  public
    { public declarations }
  end;

var
  Form1: TForm1;

type
  TMySkymp = class (TSkymp)
  public
    procedure OutStr(s: ansistring); override;
    procedure ProgressCallback(p: float); override;
  end;

implementation

{$R *.lfm}

  { TForm1 }

  procedure TForm1.LoadButtonClick(Sender: TObject);
  begin
    Memo1.Lines.Clear;
    OpenDialog1.InitialDir:= dir;
    if OpenDialog1.Execute then begin
      dir:= ExtractFilePath(OpenDialog1.FileName);
      filename:= OpenDialog1.FileName;

      Run;
    end
    else
      Memo1.Lines.Add('File open dialog did not succeed.');
  end;

  procedure TForm1.ButtonExecuteClick(Sender: TObject);
  begin
    Memo1.Lines.Clear;
    Run;
  end;

  procedure TForm1.FormCreate(Sender: TObject);
  begin
    ini:= TIniFile.Create(ChangeFileExt(ParamStr(0), '.ini'));
    dir:= ini.ReadString('session', 'dir', '');
    Width:= max(300, ini.ReadInteger('window', 'width',500));
    Height:= max (300, ini.ReadInteger('window', 'height',400));
  end;

  procedure TForm1.FormDestroy(Sender: TObject);
  begin
    ini.WriteString('session', 'dir', dir);
    ini.WriteInteger('window', 'width', Width);
    ini.WriteInteger('window', 'height', height);
    ini.Free;
  end;

  procedure TMySkymp.OutStr(s: ansistring);
  begin
    Form1.Memo1.Lines.Add(s);
    Application.ProcessMessages;
  end;


  procedure TMySkymp.ProgressCallback(p: float);
  var curprogress: integer;
  begin
    curprogress:= round(p * 1000);
    if curprogress = prevprogress then Exit;
    prevprogress:= curprogress;
    Form1.ProgressBar1.Position:= curprogress;
    Application.ProcessMessages;
  end;

  procedure TForm1.Run;
//  GetLineInfo(addr:ptruint;var func,source:string;var line:longint) : boolean;
  var
    func,source: shortstring;
    line: longint;
  begin
    try
      try
        ButtonRepeat.Enabled:= false;
        LoadButton.Enabled:= false;
        Application.ProcessMessages;

        skymp:= TMySkymp.Create(filename);
        if not skymp.execute
          then Memo1.Lines.Add('FAILED.');
        skymp.Free;

      except
        Memo1.Lines.Add('');
        Memo1.Lines.Add('---------------------------------');
        Memo1.Lines.Add((ExceptObject as Exception).Message);
        if not ((ExceptObject as Exception) is ESkympDie) then begin
          Memo1.Lines.Add('');
          if GetLineInfo(ptruint(ExceptAddr), func, source, line) then begin
            Memo1.Lines.Add(ExceptObject.Classname + ' at ' + ExtractFileName(source) + ':' + IntToStr(line) + ' (' + LowerCase(func) + ')');
          end
          else
            Memo1.Lines.Add(ExceptObject.Classname + ' at ' + IntToHex(ptruint(ExceptAddr), 8));
        end;
      end;
    finally
      LoadButton.Enabled:= true;
      ButtonRepeat.Enabled:= true;
    end;
  end;

end.

